@@ -44,7 +44,8 @@ get_cert(session *ssn)
 	if (!((verify == X509_V_OK) ||
 	    (verify == X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT) ||
 	    (verify == X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY))) {
-		error("certificate verification failed; %d\n", verify);
+		error("certificate verification failed; %s\n",
+		      X509_verify_cert_error_string(verify));
 		goto fail;
 	}
 
@@ -63,7 +64,7 @@ get_cert(session *ssn)
 				goto fail;
 			break;
 		case -1:
-			error("certificate mismatch occured\n");
+			error("certificate mismatch occurred\n");
 			goto fail;
 		}
 	}
